@@ -29,6 +29,7 @@
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include "huaweifusionsolardiscovery.h"
+#include "huaweismartloggerdiscovery.h"
 #include "integrationpluginhuawei.h"
 #include "plugininfo.h"
 
@@ -43,6 +44,7 @@ IntegrationPluginHuawei::IntegrationPluginHuawei()
 void IntegrationPluginHuawei::discoverThings(ThingDiscoveryInfo *info)
 {
     if (info->thingClassId() == huaweiFusionSolarInverterThingClassId) {
+
         if (!hardwareManager()->networkDeviceDiscovery()->available()) {
             qCWarning(dcHuawei()) << "The network discovery is not available on this platform.";
             info->finish(Thing::ThingErrorUnsupportedFeature, QT_TR_NOOP("The network device discovery is not available."));
@@ -104,6 +106,61 @@ void IntegrationPluginHuawei::discoverThings(ThingDiscoveryInfo *info)
         // Start the discovery process
         discovery->startDiscovery();
 
+    } else if (info->thingClassId() == huaweiSmartLoggerThingClassId) {
+
+        if (!hardwareManager()->networkDeviceDiscovery()->available()) {
+            qCWarning(dcHuawei()) << "The network discovery is not available on this platform.";
+            info->finish(Thing::ThingErrorUnsupportedFeature, QT_TR_NOOP("The network device discovery is not available."));
+            return;
+        }
+
+        // Create a discovery with the info as parent for auto deleting the object once the discovery info is done
+        HuaweiSmartLoggerDiscovery *discovery = new HuaweiSmartLoggerDiscovery(hardwareManager()->networkDeviceDiscovery(), 502, info);
+        connect(discovery, &HuaweiSmartLoggerDiscovery::discoveryFinished, info, [this, info, discovery](){
+            foreach (const HuaweiSmartLoggerDiscovery::Result &result, discovery->results()) {
+
+                QString name = QT_TR_NOOP("Huawei SmartLogger");
+
+                QString description;
+                switch (result.networkDeviceInfo.monitorMode()) {
+                case NetworkDeviceInfo::MonitorModeMac:
+                    description += " MAC: " + result.networkDeviceInfo.macAddressInfos().constFirst().macAddress().toString() +
+                                   " - " + result.networkDeviceInfo.address().toString();
+                    break;
+                case NetworkDeviceInfo::MonitorModeHostName:
+                    description += " Host name: " + result.networkDeviceInfo.hostName() +
+                                   " - " + result.networkDeviceInfo.address().toString();
+                    break;
+                case NetworkDeviceInfo::MonitorModeIp:
+                    description += " IP: " + result.networkDeviceInfo.address().toString();
+                    break;
+                }
+
+                ThingDescriptor descriptor(huaweiSmartLoggerThingClassId, name, description) ;
+                qCDebug(dcHuawei()) << "Discovered:" << descriptor.title() << descriptor.description();
+
+                ParamList params;
+                params << Param(huaweiSmartLoggerThingMacAddressParamTypeId, result.networkDeviceInfo.thingParamValueMacAddress());
+                params << Param(huaweiSmartLoggerThingHostNameParamTypeId, result.networkDeviceInfo.thingParamValueHostName());
+                params << Param(huaweiSmartLoggerThingAddressParamTypeId, result.networkDeviceInfo.thingParamValueAddress());
+                descriptor.setParams(params);
+
+                // Check if we already have set up this device
+                Thing *existingThing = myThings().findByParams(params);
+                if (existingThing) {
+                    qCDebug(dcHuawei()) << "This smartlogger already exists in the system:" << result.networkDeviceInfo;
+                    descriptor.setThingId(existingThing->id());
+                }
+
+                info->addThingDescriptor(descriptor);
+            }
+
+            info->finish(Thing::ThingErrorNoError);
+        });
+
+        // Start the discovery process
+        discovery->startDiscovery();
+
     } else if (info->thingClassId() == huaweiRtuInverterThingClassId) {
         qCDebug(dcHuawei()) << "Discovering modbus RTU resources...";
         if (hardwareManager()->modbusRtuResource()->modbusRtuMasters().isEmpty()) {
@@ -182,6 +239,49 @@ void IntegrationPluginHuawei::setupThing(ThingSetupInfo *info)
         return;
     }
 
+    if (thing->thingClassId() == huaweiSmartLoggerThingClassId) {
+
+        // Handle reconfigure
+        if (m_smartLoggerConnections.contains(thing))
+            m_smartLoggerConnections.take(thing)->deleteLater();
+
+        if (m_monitors.contains(thing))
+            hardwareManager()->networkDeviceDiscovery()->unregisterMonitor(m_monitors.take(thing));
+
+        // Create a monitor so we always get the correct IP in the network and see if the device is reachable without polling on our own
+        NetworkDeviceMonitor *monitor = hardwareManager()->networkDeviceDiscovery()->registerMonitor(thing);
+        if (!monitor) {
+            qCWarning(dcHuawei()) << "Failed to set up SmartLogger because the params are incomplete for creating a monitor:" << thing->params();
+            info->finish(Thing::ThingErrorInvalidParameter, QT_TR_NOOP("The parameters are incomplete. Please reconfigure the device to fix this."));
+            return;
+        }
+
+        m_monitors.insert(thing, monitor);
+        connect(info, &ThingSetupInfo::aborted, monitor, [=](){
+            hardwareManager()->networkDeviceDiscovery()->unregisterMonitor(m_monitors.take(thing));
+        });
+
+        // Continue with setup only if we know that the network device is reachable
+        if (info->isInitialSetup()) {
+            if (monitor->reachable()) {
+                setupSmartLogger(info);
+            } else {
+                // otherwise wait until we reach the networkdevice before setting up the device
+                qCDebug(dcHuawei()) << "Network device" << thing->name() << "is not reachable yet. Continue with the setup once reachable.";
+                connect(monitor, &NetworkDeviceMonitor::reachableChanged, info, [=](bool reachable){
+                    if (reachable) {
+                        qCDebug(dcHuawei()) << "Network device" << thing->name() << "is now reachable. Continue with the setup...";
+                        setupSmartLogger(info);
+                    }
+                });
+            }
+        } else {
+            setupSmartLogger(info);
+        }
+
+        return;
+    }
+
     if (thing->thingClassId() == huaweiRtuInverterThingClassId) {
 
         uint address = thing->paramValue(huaweiRtuInverterThingSlaveAddressParamTypeId).toUInt();
@@ -399,7 +499,10 @@ void IntegrationPluginHuawei::setupThing(ThingSetupInfo *info)
 
 void IntegrationPluginHuawei::postSetupThing(Thing *thing)
 {
-    if (thing->thingClassId() == huaweiFusionSolarInverterThingClassId || thing->thingClassId() == huaweiRtuInverterThingClassId) {
+    if (thing->thingClassId() == huaweiFusionSolarInverterThingClassId ||
+        thing->thingClassId() == huaweiRtuInverterThingClassId ||
+        thing->thingClassId() == huaweiSmartLoggerThingClassId) {
+
         if (!m_pluginTimer) {
             m_pluginTimer = hardwareManager()->pluginTimerManager()->registerTimer(2);
             connect(m_pluginTimer, &PluginTimer::timeout, this, [this] {
@@ -409,6 +512,12 @@ void IntegrationPluginHuawei::postSetupThing(Thing *thing)
                     }
                 }
 
+                foreach(HuaweiSmartLoggerModbusTcpConnection *connection, m_smartLoggerConnections) {
+                    if (connection->reachable()) {
+                        connection->update();
+                    }
+                }
+
                 foreach(HuaweiModbusRtuConnection *connection, m_rtuConnections) {
                     connection->update();
                 }
@@ -438,6 +547,12 @@ void IntegrationPluginHuawei::thingRemoved(Thing *thing)
         delete connection;
     }
 
+    if (m_smartLoggerConnections.contains(thing)) {
+        HuaweiSmartLogger *connection = m_smartLoggerConnections.take(thing);
+        connection->disconnectDevice();
+        delete connection;
+    }
+
     if (m_rtuConnections.contains(thing)) {
         m_rtuConnections.take(thing)->deleteLater();
     }
@@ -662,6 +777,106 @@ void IntegrationPluginHuawei::setupFusionSolar(ThingSetupInfo *info)
     connection->connectDevice();
 }
 
+void IntegrationPluginHuawei::setupSmartLogger(ThingSetupInfo *info)
+{
+    Thing *thing = info->thing();
+    NetworkDeviceMonitor *monitor = m_monitors.value(thing);
+    uint port = thing->paramValue(huaweiSmartLoggerThingPortParamTypeId).toUInt();
+    quint16 meterSlaveId = thing->paramValue(huaweiSmartLoggerThingMeterSlaveIdParamTypeId).toUInt();
+
+    qCDebug(dcHuawei()) << "Setup connection to smarlogger on" << monitor->networkDeviceInfo().address().toString() << port << "Meter slave ID" << meterSlaveId;
+
+    HuaweiSmartLogger *connection = new HuaweiSmartLogger(monitor->networkDeviceInfo().address(), port, meterSlaveId, this);
+    connect(info, &ThingSetupInfo::aborted, connection, [this, connection, thing](){
+        connection->deleteLater();
+        m_smartLoggerConnections.remove(thing);
+    });
+
+    m_smartLoggerConnections.insert(thing, connection);
+    info->finish(Thing::ThingErrorNoError);
+
+    qCDebug(dcHuawei()) << "Setup huawei smart logger finished successfully";
+
+    // Reset history, just incase
+    m_inverterEnergyProducedHistory[thing].clear();
+
+    // Add the current value to the history
+    evaluateEnergyProducedValue(thing, thing->stateValue(huaweiSmartLoggerTotalEnergyProducedStateTypeId).toFloat());
+
+    connect(connection, &HuaweiSmartLogger::reachableChanged, thing, [=](bool reachable){
+        qCDebug(dcHuawei()) << "Reachable changed to" << reachable << "for" << thing;
+        thing->setStateValue("connected", reachable);
+        foreach (Thing *childThing, myThings().filterByParentId(thing->id())) {
+            childThing->setStateValue("connected", reachable);
+
+            if (!reachable) {
+                // Set power values to 0 since we don't know what the current value is
+                if (childThing->thingClassId() == huaweiSmartLoggerThingClassId) {
+                    thing->setStateValue(huaweiSmartLoggerCurrentPowerStateTypeId, 0);
+                }
+
+                if (childThing->thingClassId() == huaweiMeterThingClassId) {
+                    thing->setStateValue(huaweiMeterCurrentPowerStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPhaseAStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPhaseBStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPhaseCStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPowerPhaseAStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPowerPhaseBStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterCurrentPowerPhaseCStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterVoltagePhaseAStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterVoltagePhaseBStateTypeId, 0);
+                    thing->setStateValue(huaweiMeterVoltagePhaseCStateTypeId, 0);
+                }
+            }
+        }
+    });
+
+    connect(monitor, &NetworkDeviceMonitor::reachableChanged, thing, [=](bool reachable){
+        if (!thing->setupComplete())
+            return;
+
+        qCDebug(dcHuawei()) << "Network device monitor for" << thing->name() << (reachable ? "is now reachable" : "is not reachable any more" );
+
+        if (reachable && !thing->stateValue("connected").toBool()) {
+            connection->modbusTcpMaster()->setHostAddress(monitor->networkDeviceInfo().address());
+            connection->connectDevice();
+        } else if (!reachable) {
+            // Note: We disable autoreconnect explicitly and we will
+            // connect the device once the monitor says it is reachable again
+            connection->disconnectDevice();
+        }
+    });
+
+    connect(connection, &HuaweiSmartLoggerModbusTcpConnection::updateFinished, thing, [this, thing, connection](){
+        qCDebug(dcHuawei()) << "Smartlogger update finished" << thing << qobject_cast<HuaweiSmartLoggerModbusTcpConnection *>(connection);
+
+        // Set inverter data
+        thing->setStateValue(huaweiSmartLoggerCurrentPowerStateTypeId, connection->inverterTotalActivePower() * -1);
+        thing->setStateValue(huaweiSmartLoggerTotalEnergyProducedStateTypeId, connection->inverterTotalEnergyProduced());
+
+        // Update the meter data
+        Thing *meterThing = myThings().filterByParentId(thing->id()).filterByThingClassId(huaweiMeterThingClassId).first();
+        if (!meterThing)
+            return;
+
+        meterThing->setStateValue(huaweiMeterCurrentPowerStateTypeId, connection->meterActivePower());
+        meterThing->setStateValue(huaweiMeterCurrentPhaseAStateTypeId, connection->meterCurrentPhaseA() * -1);
+        meterThing->setStateValue(huaweiMeterCurrentPhaseBStateTypeId, connection->meterCurrentPhaseB()* -1);
+        meterThing->setStateValue(huaweiMeterCurrentPhaseCStateTypeId, connection->meterCurrentPhaseC()* -1);
+        meterThing->setStateValue(huaweiMeterCurrentPowerPhaseAStateTypeId, connection->meterPowerPhaseA());
+        meterThing->setStateValue(huaweiMeterCurrentPowerPhaseBStateTypeId, connection->meterPowerPhaseB());
+        meterThing->setStateValue(huaweiMeterCurrentPowerPhaseCStateTypeId, connection->meterPowerPhaseC());
+        meterThing->setStateValue(huaweiMeterVoltagePhaseAStateTypeId, connection->meterVoltagePhaseA());
+        meterThing->setStateValue(huaweiMeterVoltagePhaseBStateTypeId, connection->meterVoltagePhaseB());
+        meterThing->setStateValue(huaweiMeterVoltagePhaseCStateTypeId, connection->meterVoltagePhaseC());
+
+        meterThing->setStateValue(huaweiMeterTotalEnergyProducedStateTypeId, connection->meterNegativeActiveElectricity());
+        meterThing->setStateValue(huaweiMeterTotalEnergyConsumedStateTypeId, connection->meterPositiveActiveElectricity());
+    });
+
+    connection->connectDevice();
+}
+
 void IntegrationPluginHuawei::evaluateEnergyProducedValue(Thing *inverterThing, float energyProduced)
 {
     // Add the value to our small history
@@ -669,7 +884,7 @@ void IntegrationPluginHuawei::evaluateEnergyProducedValue(Thing *inverterThing,
 
     int historyMaxSize  = 3;
     int absurdlyHighValueLimit= 25000000;
-    int historySize = m_inverterEnergyProducedHistory.value(inverterThing).count();
+    int historySize = m_inverterEnergyProducedHistory.value(inverterThing).length();
 
     if (historySize > historyMaxSize) {
         m_inverterEnergyProducedHistory[inverterThing].removeFirst();
